package laboratoire4;

import java.util.Collection;

public class BoardEvaluator {
    public static int evaluate(MovingBoard game) {
        MovingPawn[][] board = game.getBoard();
        Collection<MovingPawn> maxPawns = game.getMaxPawns();
        Collection<MovingPawn> minPawns = game.getMinPawns();

        return evaluatePlayer(board, maxPawns, minPawns) - evaluatePlayer(board, minPawns, maxPawns);
    }

    //region Full Board
    private static int evaluatePlayer(MovingPawn[][] board, Collection<MovingPawn> maxPawns, Collection<MovingPawn> minPawns) {
        if (minPawns.size() == 0) {
            return Integer.MAX_VALUE;
        }

        int score = evaluateMaxPawnCountScore(maxPawns, minPawns);

        for (MovingPawn pawn : maxPawns) {
            score += evaluatePawn(board, pawn);
        }

        return score;
    }

    private static int evaluateMaxPawnCountScore(Collection<MovingPawn> maxPawns, Collection<MovingPawn> minPawns) {
        return evaluatePawnCountScore(minPawns) - evaluatePawnCountScore(maxPawns);
    }

    // Augmente beaucoup le score à partir de 10 pions capturés
    // f(x) = 0.001x^4, f(1) = 0, f(10) = 10, f(16) = 65.54
    private static int evaluatePawnCountScore(Collection<MovingPawn> pawns) {
        return (int) (0.001f * Math.pow(16 - pawns.size(), 4)) * 100;
    }
    //endregion

    //region Board
    private static int evaluatePawn(MovingPawn[][] board, MovingPawn pawn) {
        if (pawn.getRow() == pawn.getPlayer().getGoal()) {
            return Integer.MAX_VALUE;
        }

        // Favorise les pièces qui peuvent gagner au prochain tour
        if (pawn.getRow() + pawn.getDirection() == pawn.getPlayer().getGoal()) {
            return Integer.MAX_VALUE / 2;
        }

        int score = 0;

        score += evaluateHomePawnScore(pawn);
        score += evaluatePushedScore(pawn);
        score += evaluateValidMovesScore(board, pawn);
        score += evaluateAttackScore(board, pawn);

        return score;
    }

    private static int evaluateHomePawnScore(MovingPawn pawn) {
        return pawn.getRow() == pawn.getPlayer().getHome() ? 50 : 0;
    }

    private static int evaluatePushedScore(MovingPawn pawn) {
        return pawn.isPusher() ? 0 : 50;
    }

    private static int evaluateValidMovesScore(MovingPawn[][] board, MovingPawn pawn) {
        int score = 0;

        if (pawn.isMoveValid(board, Pawn.PawnMovement.LEFT_DIAGONAL)) {
            score += 50;
        }
        if (pawn.isMoveValid(board, Pawn.PawnMovement.STRAIGHT)) {
            score += 50;
        }
        if (pawn.isMoveValid(board, Pawn.PawnMovement.RIGHT_DIAGONAL)) {
            score += 50;
        }

        return score;
    }

    private static int evaluateAttackScore(MovingPawn[][] board, MovingPawn pawn) {
        int score = 0;
        int row = pawn.getRow();
        int col = pawn.getCol();

        if (pawn.isMoveValid(board, Pawn.PawnMovement.LEFT_DIAGONAL)) {
            MovingPawn destPawn = board[row + pawn.getDirection()][col - 1];

            if (destPawn != null && destPawn.getPlayer() != pawn.getPlayer()) {
                score += 100;
            }
        }

        if (pawn.isMoveValid(board, Pawn.PawnMovement.RIGHT_DIAGONAL)) {
            MovingPawn destPawn = board[row + pawn.getDirection()][col + 1];

            if (destPawn != null && destPawn.getPlayer() != pawn.getPlayer()) {
                score += 100;
            }
        }

        // Favorise les attaques de notre côté
        if ((pawn.getPlayer() == Player.RED && pawn.getRow() < 4) ||
                (pawn.getPlayer() == Player.BLACK && pawn.getRow() >= 4)) {
            score *= 2;
        }

        return score;
    }
    //endregion
}
